﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Pick_Entity
{
  public partial class Form1 : Form
  {
    F_LCEVENT LcEvent;
    IntPtr hLcWnd, hDrw, hBlock;

    public Form1()
    {
      InitializeComponent();
    }

    private void Form1_Load(object sender, EventArgs e)
    {
      IntPtr hObj0 = new IntPtr(0);
      Lcad.PropPutStr(hObj0, Lcad.LC_PROP_G_REGCODE, "4e45");
      // define event procedure
      LcEvent = new F_LCEVENT(Form1.LcEventProc);
      Lcad.EventSetProc(Lcad.LC_EVENT_PICKENT, LcEvent, 0, hObj0);
      // initialize Litecad dll
      Lcad.Initialize();

      // create CAD window
      // the size will be set in Form1_Resize()
      hLcWnd = Lcad.CreateWindow(Handle, Lcad.LC_WS_DEFAULT);

      Form1_Resize(sender, e);

      // create a drawing object
      hDrw = Lcad.CreateDrawing();
      Lcad.DrwNew(hDrw, "", hLcWnd);

      // get the block linked to the graphics window
      hBlock = Lcad.PropGetHandle(hLcWnd, Lcad.LC_PROP_WND_BLOCK);
      // add entities
      Lcad.BlockAddCircle(hBlock, 0, 0, 20, false);
      Lcad.BlockAddCircle(hBlock, 50, 0, 20, false);
      Lcad.BlockAddCircle(hBlock, 50, 50, 20, false);
      Lcad.BlockAddCircle(hBlock, 0, 50, 20, false);
      Lcad.BlockAddCircle(hBlock, 25, 25, 5, false);
      Lcad.BlockAddRect(hBlock, 0, 0, 20, 10, 0, false);
      Lcad.BlockAddRect(hBlock, 50, 0, 20, 10, 0, false);
      Lcad.BlockAddRect(hBlock, 50, 50, 20, 10, 0, false);
      Lcad.BlockAddRect(hBlock, 0, 50, 20, 10, 0, false);
      Lcad.BlockAddRect(hBlock, 25, 25, 30, 30, 0, false);
      // update 
      Lcad.BlockUpdate(hBlock, true, hObj0);

      // zoom extents
      Lcad.WndZoomRect(hLcWnd, 0, 0, 0, 0);
      Lcad.WndZoomScale(hLcWnd, 0.8);
      Lcad.WndSetFocus(hLcWnd);

      // disable select by rect
      Lcad.PropPutBool(hLcWnd, Lcad.LC_PROP_SEL_PICKBYRECT, false);
    }

    private void Form1_Resize(object sender, EventArgs e)
    {
      int x, y, w, h;

      // "CAD window" position
      x = 0;
      y = 0;
      w = ClientSize.Width;
      h = ClientSize.Height;
      Lcad.WndResize(hLcWnd, x, y, w, h);
    }

    public static void LcEventProc(IntPtr hEvent)
    {
      int EventType;
      EventType = Lcad.PropGetInt(hEvent, Lcad.LC_PROP_EVENT_TYPE);
      switch (EventType)
      {
        case Lcad.LC_EVENT_PICKENT: OnPickEnt(hEvent); break;
      }
    }

    // this event is called when user clicks on some entity
    // but LiteCAD don't yet select it
    public static void OnPickEnt(IntPtr hEvent)
    {
      int Col, EntType;
      IntPtr hEnt = Lcad.PropGetHandle(hEvent, Lcad.LC_PROP_EVENT_ENT);
      IntPtr hLcWnd = Lcad.PropGetHandle(hEvent, Lcad.LC_PROP_EVENT_WND);

      EntType = Lcad.PropGetInt(hEnt, Lcad.LC_PROP_ENT_TYPE);
      if (EntType == Lcad.LC_ENT_CIRCLE)
      {
        // modify the circle radius by dialog
        DgSetRadius(hEnt, hLcWnd);
      }
      else
      {
        // change the entity color
        Col = Lcad.PropGetInt(hEnt, Lcad.LC_PROP_ENT_COLORT);
        if (Col == 255)
        {
          Lcad.PropPutInt(hEnt, Lcad.LC_PROP_ENT_COLORT, 65280);
        }
        else
        {
          Lcad.PropPutInt(hEnt, Lcad.LC_PROP_ENT_COLORT, 255);
        }
      }
      Lcad.WndRedraw(hLcWnd);

      // If return 1 then LiteCAD will not select the picked entity
      Lcad.EventReturnCode(1);
    }

    public static bool DgSetRadius(IntPtr hCircle, IntPtr hLcWnd)
    {
      string StrVal;
      double R;
      IntPtr hObj0 = new IntPtr(0);
      IntPtr hBlock = Lcad.PropGetHandle(hCircle, Lcad.LC_PROP_ENT_BLOCK);

      R = Lcad.PropGetFloat(hCircle, Lcad.LC_PROP_CIRC_R);
      StrVal = R.ToString();
      Lcad.PropPutStr(hObj0, Lcad.LC_PROP_G_DLGVAL, StrVal);
      if (Lcad.DgGetValue(hLcWnd, -1, -1, "Circle", "Enter Radius:"))
      {

        // StrVal = Lcad.PropGetStr(0, Lcad.LC_PROP_G_DLGVAL);
        // if Lcad.PropGetStr return wrong value, then use the following function
        StrVal = LcPropGetStr(hObj0, Lcad.LC_PROP_G_DLGVAL, 32);

        R = Convert.ToDouble(StrVal);
        if (R > 0)
        {
          Lcad.PropPutFloat(hCircle, Lcad.LC_PROP_CIRC_R, R);
          Lcad.EntUpdate(hCircle);
          Lcad.BlockUpdate(hBlock, false, hObj0);
        }
        return true;
      }
      return false;
    }

    // use this function instead of Lcad.PropGetStr, if Lcad.PropGetStr returns wrong value
    public static string LcPropGetStr(IntPtr hObj, int idProp, int MaxChars)
    {
      char[] Buf = new char[MaxChars];
      int i, ch;

      Lcad.PropGetStr(hObj, idProp);
      i = 0;
      Array.Clear(Buf, 0, MaxChars);
      do
      {
        ch = Lcad.PropGetChar(i);
        Buf[i] = Convert.ToChar(ch);
        i += 1;
      } while (ch != 0);
      return new string(Buf);
    }

  }
}
